from entities.descriptors import SwitchDescriptor
from firmwareconfig import FirmwareConfig as fwcfg


class SwitchList:
    """A class to define the list of switches of the board"""

    # this class can be actually created only once
    _ready = False

    def __init__(self):
        assert self._ready is False, "Class {} should be created only once!".format(self.__class__.__name__)
        self._create()
        self._ready = True

    def count(self):
        return len(self.vars)

    def entries(self):
        return self.vars

    def _create(self):
        self.vars = list()
        self.vars.append(
            SwitchDescriptor(
                enum_index=1,
                firmware_config=fwcfg,
                name='SWITCH_VEOC_RETURN',
                ioname='DOUT_375V_RTN_SWITCH_ON_OFF',
                description='DU backbone ON/OFF return line control (high=ON)',
            )
        )
        self.vars.append(
            SwitchDescriptor(
                enum_index=2,
                firmware_config=fwcfg,
                name='SWITCH_VEOC_DIRECT',
                ioname='DOUT_375V_FWR_SWTCH_ON_OFF',
                description='DU backbone ON/OFF forward line control (high=ON)',
            )
        )
        self.vars.append(
            SwitchDescriptor(
                enum_index=3,
                firmware_config=fwcfg,
                name='SWITCH_12V',
                ioname='DOUT_ISO_12V_SWITCH_ON_OFF',
                description='DC-DC TDK 5/12V ISO ON/OFF control (high=ON)',
            )
        )
        self.vars.append(
            SwitchDescriptor(
                enum_index=4,
                firmware_config=fwcfg,
                name='SWITCH_LBL',
                ioname='DOUT_LBL_SWITCH_ON_OFF',
                description='12V ISO LBL power ON/OFF control (high=ON)',
            )
        )
        self.vars.append(
            SwitchDescriptor(
                enum_index=5,
                firmware_config=fwcfg,
                name='SWITCH_HYDRO',
                ioname='DOUT_HYDRO_SWITCH_ON_OFF',
                description='12V ISO HYDRO power ON/OFF control (high=ON)',
            )
        )
        self.vars.append(
            SwitchDescriptor(
                enum_index=6,
                firmware_config=fwcfg,
                name='SWITCH_GLRA',
                ioname='DOUT_GLR_A_SWITCH_ON_OFF',
                description='12V GLENAIR-A power ON/OFF control (high=ON)',
            )
        )
        self.vars.append(
            SwitchDescriptor(
                enum_index=7,
                firmware_config=fwcfg,
                name='SWITCH_GLRB',
                ioname='DOUT_GLR_B_SWITCH_ON_OFF',
                description='12V GLENAIR-B power ON/OFF control (high=ON)',
            )
        )
        self.vars.append(
            SwitchDescriptor(
                enum_index=8,
                firmware_config=fwcfg,
                name='SWITCH_PWB',
                ioname='DOUT_PWB_SWITCH_ON_OFF',
                description='12V  POWER BOARD power ON/OFF control (high=ON)',
            )
        )


if __name__ == '__main__':
    switches = SwitchList()
    print switches.vars[0].name
