#!/usr/bin/python
"""
A minimal utility to continuously read all sensors
"""

import sys
import os
sys.path.insert(0, '../codegen')


import commands
from utils import mls
from analogvariables import *
from bpsentities.descriptors import PayloadFieldEnum, PayloadFieldU8, PayloadFieldU16, PayloadFieldU32
from commutils import *
import serial
from time import time


ASK_CONFIRM = False
SHOW_SENT_DATA = False
SHOW_RECEIVED_DATA = False
UART_DEV = '/dev/ttyUSB1'


def ask_confirm():
    answer = ""
    while answer not in ["y", "n"]:
        answer = raw_input("Confirm [Y/N]? ").lower()
    return answer == "y"


if __name__ == '__main__':
    # command line options
    # example
    # python sendcommand.py COMMAND_NAME payload0 payload1 ...
    # COMMAND_NAME is the human readable name of the command (e.g.: )

    # redirect stdout to null in order to avoid the warnings during object creation
    stdout = sys.stdout
    f = open(os.devnull, 'w')
    sys.stdout = f

    switch_list = commands.SwitchList()
    analog_variable_list = commands.AnalogVariableList()
    digital_variable_list = commands.DigitalVariableList()
    user_pin_list = commands.UserPinList()
    commands = commands.CommandList(switch_list, analog_variable_list, digital_variable_list, user_pin_list)

    # restore normal stdout
    sys.stdout = stdout

    # open the serial port
    ser = serial.Serial(UART_DEV, baudrate=19200, bytesize=serial.EIGHTBITS, parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE, timeout=0.5)

    command_name = 'SENSOR_VALUES_GETALL'
    filtered_commands = [command for command in commands.entries() if command.name.startswith(command_name)]
    command = filtered_commands[0]

    # expected number of request fields:
    req_field_count = len(command.request_payload)

    # prepare the payload
    payload_data = list()
    payload_field_values = list()
    rawdata = packet_create_rawdata(command.request_code, payload_data)

    while True:
        # send the message over the serial link
        ser.write(rawdata)
        print "Packet sent."
        if SHOW_SENT_DATA:
            print "Sent data are:"
            for val in rawdata:
                print '\t{}\t(ascii: {})'.format(val, chr(val))

        # wait for a response (reads up to 1000 bytes, does not parse the packet)
        print 'Waiting for response...'
        ans = ser.read(1000)
        bytes = [ord(c) for c in ans]
        print '{} bytes received.'.format(len(ans))
        if SHOW_RECEIVED_DATA:
            if len(ans):
                print 'Received data:'
                for k in range(len(ans)):
                    print '[byte #{}]\t0x{}\t(dec: {}, ascii: {})'.format(str(k), ans[k].encode('hex'), str(ord(ans[k])), ans[k])
        if not len(ans):
            print 'No response...'
            continue
            # sys.exit(0)

        # parses the response
        command_code, payload_bytes = packet_extract_data(bytes)
        print "Received raw payolad:", payload_bytes

        print "Received packet:"
        print '    Command code: {}'.format(command_code)
        # print '    Payload data: {} (raw data: {})'.format(payload_field_values, payload_data)
        print '    Response payload:'
        byte_idx = 0
        for field in command.response_payload:
            value = -1
            if isinstance(field, PayloadFieldU8):
                value = payload_bytes[byte_idx]
                byte_idx += 1
            elif isinstance(field, PayloadFieldU16):
                value = payload_bytes[byte_idx] << 8
                value += payload_bytes[byte_idx+1]
                byte_idx += 2
            elif isinstance(field, PayloadFieldU32):
                value = payload_bytes[byte_idx] << 24
                value += payload_bytes[byte_idx+1] << 16
                value += payload_bytes[byte_idx+2] << 8
                value += payload_bytes[byte_idx+3]
                byte_idx += 4
            elif isinstance(field, PayloadFieldEnum):
                value = field.get_value_by_index(payload_bytes[byte_idx])
                byte_idx += 1
            else:
                assert False, 'Unmanaged type {}'.format(field.__class__.__name__)

            print '        {} = {}'.format(field.name, value)

            t = time()
            # append read values to a file per sensor, with timestamp
            with open('outputs/continuousreads/{}.txt'.format(field.name), 'a') as f:
                data = f.write('{}\t{}\n'.format(t, value))

