#!/usr/bin/python
"""
A minimal utility to send packets to the BPS board
"""

import serial
import sys
from commutils import *

ASK_CONFIRM = False
UART_DEV = '/dev/ttyUSB0'


def ask_confirm():
    answer = ""
    while answer not in ["y", "n"]:
        answer = raw_input("Confirm [Y/N]? ").lower()
    return answer == "y"


if __name__ == '__main__':
    # command line options
    # example
    # python sendrawpacket.py 0x12 0x23 0x34
    # sends the command with code 0x12, and a payload of 2 bytes : [0x23, 0x34]
    # numbers can also be given in decimal

    assert len(sys.argv) > 1, "Command code missing"

    # open the serial port
    ser = serial.Serial(UART_DEV, baudrate=19200, bytesize=serial.EIGHTBITS, parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE, timeout=0.5)

    # first argument must be command code
    command_code = parse_num(sys.argv[1])

    # payload data are the rest of arguments
    payload_data = list()
    for k in range(2, len(sys.argv)):
        payload_data.append(parse_num(sys.argv[k]))

    print "Command code is: 0x%02x" % command_code, "(", command_code, ")"
    print "Payload data is:", ["0x%02x" % d for d in payload_data]

    if ASK_CONFIRM:
        if not ask_confirm():
            print "Operation aborted by user"
            sys.exit()

    rawdata = packet_create_rawdata(command_code, payload_data)

    # send the message over the serial link
    ser.write(rawdata)
    print "Packet sent."
    print "Sent data are:"
    for val in rawdata:
        print '\t{}\t(ascii: {})'.format(val, chr(val))

    # wait for a response (reads up to 1000 bytes, does not parse the packet)
    print 'Waiting for response...'
    ans = ser.read(1000)
    print '{} bytes received.'.format(len(ans))
    if len(ans):
        print 'Received data:'
        for k in range(len(ans)):
            print '[byte #{}]\t0x{}\t(dec: {}, ascii: {})'.format(str(k), ans[k].encode('hex'), str(ord(ans[k])), ans[k])









